# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

"""
This module contains general helper functions to aid in developing AiiDA calculations and workchains.
"""

__version__ = '0.1.1'

from ._input_validator import get_input_validator
from ._check_workchain_step import *
from . import workchain_inputs

__all__ = ['workchain_inputs'] + _input_validator.__all__ + _check_workchain_step.__all__



# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import functools
import traceback

from fsc.export import export


@export
def check_workchain_step(func):
    """
    Decorator for workchain steps that logs (and re-raises) errors occuring within that step.
    """
    @functools.wraps(func)
    def inner(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except Exception as e:
            self.report(
                '{} in {}: {}.\nTraceback:\n{}'.format(type(e).__name__, func.__name__, e, traceback.format_exc())
            )
            raise e

    return inner



# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>


"""
Contains default keyword arguments to pass classes as input to workchains.
"""

try:
    from functools import singledispatch
except ImportError:
    from singledispatch import singledispatch

import yaml
from fsc.export import export
from aiida.work import ObjectLoader

from aiida.orm.data.str import Str

__all__ = ['WORKCHAIN_INPUT_KWARGS']

_YAML_IDENTIFIER = '!!YAML!!'

@export
@singledispatch
def get_fullname(cls_obj):
    """
    Serializes an AiiDA workchain or workfunction to an AiiDA String. For workchains the class identifier is used, workfunctions are serialized in YAML format.

    :param cls_obj: Object to be serialized
    :type cls_obj: WorkChain, workfunction
    """
    try:
        return Str(ObjectLoader().identify_object(cls_obj))
    except ValueError:
        return Str(_YAML_IDENTIFIER + yaml.dump(cls_obj))


@get_fullname.register(str)
def _(cls_name):
    return Str(cls_name)

#: Keyword arguments to be passed to ``spec.input`` for serializing an input which is a class / workchain into a string.
WORKCHAIN_INPUT_KWARGS = {
    'valid_type': Str,
    'serializer': get_fullname,
}

@export
def load_object(cls_name):
    """
    Loads the workchain or workfunction from the serialized string.
    """
    cls_name_str = str(cls_name)
    try:
        return ObjectLoader().load_object(cls_name_str)
    except ValueError as err:
        if cls_name_str.startswith(_YAML_IDENTIFIER):
            return yaml.load(cls_name_str[len(_YAML_IDENTIFIER):])
        raise err



# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>


from aiida.common.exceptions import InputValidationError

__all__ = ['get_input_validator']

def get_input_validator(inputdict):
    """
    Returns a function which validates and returns a given input.

    :param inputdict: Inputs to the calculation.
    :type inputdict: dict

    Parameters to the returned validator function:

    :param name: Name of the input.
    :type name: str

    :param valid_types: Valid types for the input value.
    :type valid_types: type, or tuple(type)

    :param required: Indicates whether the input is required. The default is ``True``.
    :type required: bool

    :param default: Default value that is returned if the value is not given, and not required.
    """
    def _validate_input(name, valid_types, required=True, default=None):
        try:
            value = inputdict.pop(name)
        except KeyError:
            if required:
                raise InputValidationError("Missing required input parameter '{}'".format(name))
            else:
                value = default

        if not isinstance(valid_types, (list, tuple)):
            valid_types = [valid_types]
        if not required:
            valid_types = list(valid_types) + [type(default)]
        valid_types = tuple(valid_types)

        if not isinstance(value, valid_types):
            raise InputValidationError("Input parameter '{}' is of type '{}', but should be of type(s) '{}'".format(name, type(value), valid_types))
        return value

    return _validate_input



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>


import re
from setuptools import setup

readme = """Helper tools for developing AiiDA plugins."""

# Get the version number
with open('./aiida_tools/__init__.py') as f:
    match_expr = "__version__[^'\"]+(['\"])([^'\"]+)"
    version = re.search(match_expr, f.read()).group(2).strip()


setup(
    name='aiida-tools',
    version=version,
    url='https://aiida-tools.readthedocs.io',
    author='Dominik Gresch',
    author_email='greschd@gmx.ch',
    description=readme,
    install_requires=['aiida-core', 'fsc.export', 'pyyaml'],
    extras_require={
        ':python_version < "3"': ['singledispatch'],
        'doc': ['sphinx', 'sphinx-rtd-theme'],
    },
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Physics',
        'Development Status :: 3 - Alpha'
    ],
    license='Apache 2.0',
    packages=['aiida_tools']
)
